# Example: Control of the Tomato Spotted Wilt virus


#####################################################################
# Read in data

  tomato <- read.csv(file  = "C:\\data\\TomatoVirus.csv", stringsAsFactors = TRUE)
  head(tomato)
  tomato
  
  # If the default of "stringsAsFactors = FALSE" is used, the Control variable
  #   will need to be converted to a factor type using the following code
  tomato2 <- read.csv(file  = "C:\\data\\TomatoVirus.csv")
  tomato2$Control <- factor(tomato$Control)


#####################################################################
# Estimate model

  # Check out the Control variable
  class(tomato$Control)
  levels(tomato$Control)
  contrasts(tomato$Control)

  # Check out the Infest explanatory variable
  class(tomato$Infest)
  levels(tomato$Infest)

  # Change Infest to a factor class
  tomato$Infest <- factor(tomato$Infest)
  head(tomato)
  class(tomato$Infest)
  levels(tomato$Infest)
  contrasts(tomato$Infest)


  # Additional examples of working with the factor() function
  temp <- factor(tomato$Control, levels = c("C", "B", "N"))  # Change ordering of levels like relevel()
  levels(temp)
  temp2 <- factor(tomato$Control, labels = c("B1", "C1", "N1"))  # Change names of levels
  levels(temp2)


  # Estimate the model
  mod.fit <- glm(formula = Virus8/Plants ~ Infest + Control, family = binomial(link = logit), data = tomato,
    weights = Plants)
  summary(mod.fit)

  mod.fit$xlevels  # Another way to see the levels of categorical explanatory variables

  # Show how to find estimates of probability
  predict.data <- data.frame(Infest = c("1", "1", "1"), Control = c("B", "C", "N"))
  predict(object = mod.fit, newdata = predict.data, type = "response")

  # Estimate the model with an interaction
  mod.fit.inter <- glm(formula = Virus8/Plants ~ Infest + Control + Infest:Control, family = binomial(link = logit), data = tomato,
    weights = Plants)
  summary(mod.fit.inter)


  # LRT
  library(package = car)
  Anova(mod.fit.inter)



#####################################################################
# Odds ratios for model without interaction term

  #############################
  # Estimated OR for Infest

    exp(mod.fit$coefficients[2])
    1/exp(mod.fit$coefficients[2])
    # as.numeric(exp(mod.fit$coefficients[2]))
    # attributes(mod.fit$coefficients[2])

    # Wald interval
    exp(confint.default(object = mod.fit, parm = "Infest2", level = 0.95))
    # as.numeric(exp(confint.default(object = mod.fit, parm = "Infest2", level = 0.95)))

    # Profile likelihood ratio interval
    exp(confint(object = mod.fit, parm = "Infest2", level = 0.95))


  #############################
  # Estimated OR for Control

    # Compare control levels C and N to B
    exp(mod.fit$coefficients[3:4])
    # Control N vs. Control C
    exp(mod.fit$coefficients[4] - mod.fit$coefficients[3])


    # Using mcprofile to find confidence intervals
    # Control
    library(package = mcprofile)
    K <- matrix(data = c(0, 0,  1,  0,
                         0, 0,  0,  1,
                         0, 0, -1,  1),  nrow = 3, ncol = 4, byrow = TRUE,
                         dimnames = list(c("C vs. B", "N vs. B", "N vs. C"), c("beta0",
                         "beta1", "beta2", "beta3")))
    K
    linear.combo <- mcprofile(object = mod.fit, CM = K)
    ci.log.OR <- confint(object = linear.combo, level = 0.95, adjust = "none")
    # R correctly applies exp() to estimates and intervals because
    #   there is a exp.mcpCI() method function
    exp(ci.log.OR)
    # Alternatively, can use a data frame
    round(data.frame(OR.hat = exp(ci.log.OR$estimate),
      OR.low = exp(ci.log.OR$confint$upper),
      OR.up = exp(ci.log.OR$confint$lower)), 2)

    # Invert
    round(data.frame(OR.hat = 1/exp(ci.log.OR$estimate),
      OR.low = 1/exp(ci.log.OR$confint$upper),
      OR.up = 1/exp(ci.log.OR$confint$lower)), 2)

    # Infest
    library(package = mcprofile)
    K <- matrix(data = c(0, 1,  0,  0),  nrow = 1, ncol = 4, byrow = TRUE,
                         dimnames = list(c("Infest = 1 vs. 0")))
    K
    linear.combo <- mcprofile(object = mod.fit, CM = K)
    ci.log.OR <- confint(object = linear.combo, level = 0.95, adjust = "none")
    exp(ci.log.OR)
    # One could also just use confint() from stats here because we just have one regression parameter



    # Using emmeans to find confidence intervals
    # Control
    library(emmeans)
    calc.est <- emmeans(object = mod.fit, specs = ~ Control,
      at = list(Control = c("B", "C", "N")),
      type = "response")
    summary(object = calc.est, level = 0.95)

    calc.est <- emmeans(object = mod.fit, specs = ~ Control,
      type = "response")
    confint(object = contrast(object = calc.est, method = "pairwise"),
      adjust = "none", level = 0.95)
    confint(object = contrast(object = calc.est, method = "revpairwise"),
      adjust = "none", level = 0.95)

    # Additional approach using pairs()
    confint(object = pairs(x = calc.est, type = "response", adjust = "none"))
    confint(object = pairs(x = calc.est, type = "response", adjust = "none",
      reverse = TRUE))

    # Changed specs argument value; intervals are the same across levels of Infest
    #   This is why the original specs argument did not contain Infest
    calc.est <- emmeans(object = mod.fit, specs = ~ Control + Infest,
      type = "response")
    confint(object = contrast(object = calc.est, method = "pairwise",
      simple = list("Control")), adjust = "none", level = 0.95)

    # Infest
    calc.est.Infest <- emmeans(object = mod.fit, specs = ~ Infest,
      type = "response")
    confint(object = contrast(object = calc.est.Infest, method = "pairwise"),
      adjust = "none", level = 0.95)
    # One could also just use confint.default() from stats here because we just have one regression parameter


    # Finding the intervals without mcprofile or emmeans

    # Wald interval
    exp(confint.default(object = mod.fit, parm = c("ControlC", "ControlN"), level = 0.95))
    # Invert - be careful with ordering of limits
    1/exp(confint.default(object = mod.fit, parm = c("ControlC", "ControlN"), level = 0.95))

    # Profile LR interval
    exp(confint(object = mod.fit, parm = c("ControlC", "ControlN"), level = 0.95))

    # Wald interval for Control N vs. Control C
    beta.hat <- mod.fit$coefficients[-1]  # Matches up beta indices with [i] to help avoid mistakes
    exp(beta.hat[3] -  beta.hat[2])
    cov.mat <- vcov(mod.fit)[2:4,2:4]
    var.N.C <- cov.mat[3,3] + cov.mat[2,2] - 2*cov.mat[3,2]
    CI.betas <- beta.hat[3] - beta.hat[2] + qnorm(p = c(0.025, 0.975)) * sqrt(var.N.C)
    exp(CI.betas)

    # A little easier way for Control N vs. Control C
    tomato$Control.reorder <- relevel(x = tomato$Control, ref = "C")
    mod.fit2 <- glm(formula = Virus8/Plants ~ Infest + Control.reorder, family = binomial(link = logit), data = tomato,
      weight = Plants)
    # summary(mod.fit2)
    exp(confint.default(object = mod.fit2, parm = c("Control.reorderB", "Control.reorderN"), level = 0.95))
    exp(confint(object = mod.fit2, parm = c("Control.reorderB", "Control.reorderN"), level = 0.95))


#####################################################################
# Odds ratios for model with interaction

  #############################
  # Estimated OR for Control

    # Extract the beta^'s so that the resulting vector will have the same indices as the subscripts
    #  for the parameter estimates
    beta.hat <- mod.fit.inter$coefficients[-1]

    C.B.Infest2.0 <- exp(beta.hat[2])
    C.B.Infest2.1 <- exp(beta.hat[2] + beta.hat[4])
    N.B.Infest2.0 <- exp(beta.hat[3])
    N.B.Infest2.1 <- exp(beta.hat[3] + beta.hat[5])
    N.C.Infest2.0 <- exp(beta.hat[3] - beta.hat[2])
    N.C.Infest2.1 <- exp(beta.hat[3] + beta.hat[5] - beta.hat[2] - beta.hat[4])
    comp <- c("C vs. B, Infest2 = 0", "C vs. B, Infest2 = 1",
              "N vs. B, Infest2 = 0", "N vs. B, Infest2 = 1",
              "N vs. C, Infest2 = 0", "N vs. C, Infest2 = 1")
    data.frame(Comparison = comp,
               OR.hat = round(c(C.B.Infest2.0, C.B.Infest2.1, N.B.Infest2.0, N.B.Infest2.1,
                 N.C.Infest2.0, N.C.Infest2.1),2))


    # Using emmeans
    # library(package = emmeans)   # If not done yet
    calc.est.inter <- emmeans(object = mod.fit.inter, specs = ~ Infest + Control,
      type = "response")
    confint(object = contrast(object = calc.est.inter, method = "revpairwise",
      simple = list("Control")), adjust = "none", level = 0.95)
    # Options for control of the familywise confidence level
    #   Default = Tukey, within Infest level only
    confint(object = contrast(object = calc.est.inter, method = "revpairwise",
      simple = list("Control")), level = 0.95)
    #   Default = Bonferroni, added combine = TRUE to have control over all levels of Infest
    confint(object = contrast(object = calc.est.inter, method = "revpairwise",
      simple = list("Control"), combine = TRUE), level = 0.95)
    #   Specify Bonferroni across all Infest levels - same result as above
    confint(object = contrast(object = calc.est.inter, method = "revpairwise",
      simple = list("Control"), combine = TRUE), adjust = "Bonferroni", level = 0.95)

    # Additional approach using the pairs() function and by argument
    #   Control of the familwise confidence level can only be done within each
    #   level of Infest here due to the by argument.
    calc.est.inter <- emmeans(object = mod.fit.inter, specs = ~ Infest + Control,
      type = "response", level = 0.95)
    confint(object = pairs(x = calc.est.inter, type = "response", by = "Infest",
      adjust = "none", reverse = TRUE))


    # Profile LR using mcprofile
    # library(package = mcprofile)  # If had not done yet
    K.names <- list(comp,
              c("beta0", "beta1", "beta2", "beta3", "beta4", "beta5"))
    # The c("beta0", "beta1", "beta2", "beta3", "beta4", "beta5")) portion
    #   can also be done with
    # paste(rep("beta", times = 7), 0:6, sep = "")
    K <- matrix(data = c(0, 0,  1,  0,  0,  0,
                         0, 0,  1,  0,  1,  0,
                         0, 0,  0,  1,  0,  0,
                         0, 0,  0,  1,  0,  1,
                         0, 0, -1,  1,  0,  0,
                         0, 0, -1,  1, -1,  1),  nrow = 6, ncol = 6, byrow = TRUE,
                         dimnames = K.names)
    K
    linear.combo <- mcprofile(object = mod.fit.inter, CM = K)
    ci.log.OR <- confint(object = linear.combo, level = 0.95, adjust = "none")
    exp(ci.log.OR)

    # Bonferroni
    ci.log.OR.bon <- confint(object = linear.combo, level = 0.95, adjust = "bonferroni")
    exp(ci.log.OR.bon)

    # Single Step
    ci.log.OR.ss <- confint(object = linear.combo, level = 0.95, adjust = "single-step")
    exp(ci.log.OR.ss)
    exp(ci.log.OR.ss$confint)


    # Wald
    save.wald <- wald(object = linear.combo)
    ci.log.OR.wald <- confint(object = save.wald, level = 0.95, adjust = "none")
    exp(ci.log.OR.wald)

    # Bonferroni
    ci.wald.bon <- confint(object = save.wald, level = 0.95, adjust = "bonferroni")
    exp(ci.wald.bon)

    # Single Step
    ci.wald.ss <- confint(object = save.wald, level = 0.95, adjust = "single-step")
    exp(ci.wald.ss)



#
