# Example: Beetle egg-laying response to crowding

# Read in beetle egg mass data
eggdata <- read.csv(file = "C:\\Data\\BeetleEggCrowding.csv", stringsAsFactors = TRUE)
eggdata$females <- ifelse(test = eggdata$TRT == "I", yes = 1, no = 5)
head(eggdata)

# Getting mean rate per female for each combination of TRT and Temp
aggregate(x = NumEggs ~ TRT + Temp, data = eggdata, FUN = mean, subset = TRT == "I")
aggregate(x = NumEggs ~ TRT + Temp, data = eggdata, FUN = mean, subset = TRT == "G")$NumEggs/5

# Fit model using temperature, crowding level, and interaction
eggmod1 <- glm(NumEggs ~ Temp*TRT, family = poisson(link = "log"), offset = log(females), data = eggdata)
summary(eggmod1)

# Predicted means using observed t, one for each cage. 
round(predict(object = eggmod1, type = "response"), 2)

# Using predict() to produce one mean count for each temp and TRT combination
newdata <- data.frame(TRT = c("G", "I", "G", "I"), Temp = c(21, 21, 24, 24), females = c(5, 1, 5, 1)) 
mu.hat <- round(predict(object = eggmod1, newdata = newdata, type = "response"), 2) 
data.frame(newdata, mu.hat, rate = mu.hat/newdata$females)

# Estimated rates and Wald confidence intervals using emmeans
library(package = emmeans)

# "Usual" code produces means rather than rates
emm1 <- emmeans(object = eggmod1, specs = ~ TRT + Temp)
emm1
confint(object = emm1, type = "response")

# Adding offset=log(n) produces rates scaled to a "per n-unit" basis.
# Typical rate is per 1 unit unless otherwise specified, so log(1)=0
#  Also adding nesting=null here because the offset values are 1:1 with the TRT levels
# (females = 1 or 5 according to whether TRT is I of G)
emm1o <- emmeans(object = eggmod1, specs = ~ TRT + Temp, offset = log(1), nesting = NULL)
emm1o
confint(object = emm1o, type = "response")


# Type II ANOVA Tests
library(package = car)
Anova(eggmod1)

# Estimated mean ratios, and confidence intervals using emmeans
# Pairwise comparisons of each variable separately at each level of other variable
mean.rat1 <- contrast(emm1o, method = "pairwise", simple = list("TRT"))
confint(mean.rat1, type = "response")

# Pairwise comparisons of all combinations
mean.rat2 <- contrast(emm1o, method = "pairwise")
# Adjusts confidence levels for multiple comparisons
confint(mean.rat2, type="response")

# Comparing the two rate ratios (mimicking interaction test)
contrast(emm1o, interaction = "pairwise")

# Use mcprofile to get LR confidence intervals.

# Matrix of linear combination coefficients for predicted rates at each combination
coef.mat <- as.matrix(rbind(c(1,21,1,21), c(1,24,1,24), c(1,21,0,0), c(1,24,0,0)))

library(mcprofile)
LRCI <- mcprofile(eggmod1, CM = coef.mat)
exp(confint(LRCI, level = 0.95))
exp(confint(LRCI, level = 0.95, adjust = "none"))
wald.calc <- wald(LRCI)
exp(confint(wald.calc, level = 0.95, adjust = "none"))

emm.ci <- confint(emm1o, type="response")
wald.ci <- cbind(emm.ci$rate, emm.ci$asymp.LCL, emm.ci$asymp.UCL) 
# Plot mean rates and confidence intervals
dev.new(height = 6, width = 7, pointsize = 15)
# pdf(file = "c:\\figures\\Figure4.8BW.pdf", width = 7, height = 6, colormodel = "cmyk", pointsize = 15)   # Create plot for book
plot(x = c(21,21,24,24), y = wald.ci[,1], xlim = c(20,25.5), ylim = c(0,5), xlab = "Temperature", ylab = "Mean Egg Masses per Female")
segments(x0 = 21, x1 = 24, y0 = wald.ci[1,1], y1 = wald.ci[3,1], lty = "dotted", lwd = 2)
segments(x0 = 21, x1 = 24, y0 = wald.ci[2,1], y1 = wald.ci[4,1], lty = "solid", lwd = 2)
segments(x0 = 21, x1 = 21, y0 = wald.ci[1,2], y1 = wald.ci[1,3], lwd = 2)
segments(x0 = 24, x1 = 24, y0 = wald.ci[3,2], y1 = wald.ci[3,3], lwd = 2)
segments(x0 = 21, x1 = 21, y0 = wald.ci[2,2], y1 = wald.ci[2,3], lwd = 2)
segments(x0 = 24, x1 = 24, y0 = wald.ci[4,2], y1 = wald.ci[4,3], lwd = 2)
legend(x = 24, y = 1, legend = c("Individual", "Crowded"), cex = 0.9, lty = c("solid", "dotted"), lwd = 2, title = "TRT Level", bty = "n")
# dev.off()  # Create plot for book


# Plot mean rates and confidence intervals: Color
dev.new(height = 6, width = 7, pointsize = 15)
# pdf(file = "c:\\figures\\Figure4.8color.pdf", width = 7, height = 6, colormodel = "cmyk", pointsize = 15)   # Create plot for book
plot(x = c(21,21,24,24), y = wald.ci[,1], xlim = c(20,25.5), ylim = c(0,5), xlab = "Temperature", ylab = "Mean Egg Masses per Female")
segments(x0 = 21, x1 = 24, y0 = wald.ci[1,1], y1 = wald.ci[3,1], col = "red", lwd = 2)
segments(x0 = 21, x1 = 24, y0 = wald.ci[2,1], y1 = wald.ci[4,1], col = "blue", lwd = 2)
segments(x0 = 21, x1 = 21, y0 = wald.ci[1,2], y1 = wald.ci[1,3], col = "red", lwd = 2)
segments(x0 = 24, x1 = 24, y0 = wald.ci[3,2], y1 = wald.ci[3,3], col = "red", lwd = 2)
segments(x0 = 21, x1 = 21, y0 = wald.ci[2,2], y1 = wald.ci[2,3], col = "blue", lwd = 2)
segments(x0 = 24, x1 = 24, y0 = wald.ci[4,2], y1 = wald.ci[4,3], col = "blue", lwd = 2)
legend(x = 24, y = 1, legend = c("Individual", "Crowded"), cex = 0.9, col = c("red", "blue"), lwd = 2, title = "TRT Level", bty = "n")
# dev.off()  # Create plot for book


############################################################
# Additional coding not discussed in the book
# Wald inferences using multcomp package
library(package = multcomp)
wald <- glht(eggmod1, linfct = coef.mat)
# Defaults use multiplicity adjustment for simultaneous confidence level
summary(wald)
round(exp(confint(wald)$confint), digits = 2)
# Options to get unadjusted (univariate) tests and CIs
summary(wald, test = univariate())
wald.ci <- exp(confint(wald, calpha = qnorm(0.975))$confint)
round(wald.ci, digits = 2)


