# Example: Placekicking, LASSO

placekick <- read.csv(file = "C:\\data\\Placekick.csv")
head(placekick)
tail(placekick)

library(glmnet)

# Fit LASSO by glmnet(y = , x = ). Gaussian is default, but other families are available 
# Function produces series of fits for many values of lambda. 
# glmnet() requires x to be in matrix class, so saving out 
#  the separate variables to be used as Y and X as new objects.
#  Note that any categorical explanatory variables need to be written out as
#  indicators, where they could be chosen separately into the model.  
#  There is also a "group LASSO" that allows simultaneous inclusion of all 
#  of these indicators. 

yy <- as.matrix(placekick[,9])
xx <- as.matrix(placekick[,1:8])

# Basic LASSO fit, showing results for many lambda values.
lasso.fit <- glmnet(y = yy, x = xx, family = "binomial")

# List out coefficients for each lambda 
round(coef(lasso.fit), digits = 3)
# Adding lambda values to output 
las.lambda <- lasso.fit$lambda
length(las.lambda)
# Matrix(x,sparse = TRUE) converts zeroes on matrix x into dots for nicer print
las.coefs <- Matrix(rbind(as.matrix(coef(lasso.fit)), las.lambda), sparse = TRUE)
#splitting up range of columns for better printing) 
round(las.coefs[,c(1:5)], digits = 4)
round(las.coefs[,c(21:25)], digits = 4)
round(las.coefs[,c(60:63)], digits = 4)

# cv.glmnet() uses crossvalidation to estimate optimal lambda
# Fix seed for crossvalidation, so the book's results can be duplicated
set.seed(27498272)
cv.lasso.fit <- cv.glmnet(y = yy, x = xx, family = "binomial")

# Default plot method for cv.lasso() produces CV errors 
#   with error bars at +/- 1 SE at each lambda. 
dev.new(width = 7, height = 6, pointsize = 12)
# pdf(file = "c:\\figures\\Figure5.1color.pdf", width = 7, height = 6, colormodel = "cmyk")   # Create plot for book
plot(cv.lasso.fit)
# dev.off()  # Create plot for book
# Print out coefficients at optimal lambda using 1SE rule
coef(cv.lasso.fit) 
# Another way to do this
coef(lasso.fit,s = cv.lasso.fit$lambda.1se) 
# Using minimum CV lambda instead yields larger, less shrunk model
coef(lasso.fit,s = cv.lasso.fit$lambda.min) 
# Predicted response values use 1SE optimal lambda, can control with s= argument
predict.las <- predict(cv.lasso.fit, newx = xx, type = "response")
head(cbind(placekick$distance, round(predict.las, digits = 3)))


#########################
# Relaxed Lasso
# The cv.glmnet() version of relaxed LASSO is a simple 
#   approximation that finds both the LASSO parameter 
#   estimates and the MLEs for the set of variables selected 
#   by each value of lambda.  It then takes a weighted 
#   average of these two sets of estimates for separately 
#   for each lambda.  The weights are controlled by a second 
#   parameter, gamma, which is set to 0 (full MLE), 0.25, 0.5, 
#   0.75, and 1 (full LASSO). 

set.seed(233335441)
cv.lasso.rlx <- cv.glmnet(y = yy, x = xx, family = "binomial", relax = TRUE)

# Default plot method for cv.lasso() produces CV errors +/- 1 SE at each lambda. 
dev.new(width = 7, height = 6, pointsize = 12)
plot(cv.lasso.rlx)

# 
# Print out coefficients at optimal lambda and gamma using 1SE rule
cv.lasso.rlx
coef(cv.lasso.rlx) 
# Another way to do this, comparing LASSO to MLE for two chosen lambdas
coef(cv.lasso.rlx, s = "lambda.1se", gamma = 0) 
coef(cv.lasso.rlx, s = "lambda.1se", gamma = 1) 
# Using minimum CV lambda instead yields larger, less shrunk model
coef(cv.lasso.rlx, s = "lambda.min", gamma = 0) 
coef(cv.lasso.rlx, s = "lambda.min", gamma = 1) 
# Predicted response values use 1SE optimal lambda and gamma.
#   Can control with s= and gamma= arguments
predict.las.r1 <- predict(cv.lasso.rlx, newx = xx, type = "response")
predict.las.rm <- predict(cv.lasso.rlx, newx = xx, type = "response", s = "lambda.min", gamma = 0)
head(cbind(placekick$distance, placekick$PAT, round(cbind(predict.las.r1, predict.las.rm), digits = 3)))

