# Placekicking, model build and interpret


# Note that the version of the data set used here includes 13 additional
#  observations and a few additional variables
placekick.mb1 <- read.csv("C:\\data\\PlacekickMB.csv")
head(placekick.mb1)
tail(placekick.mb1)
# sum(is.na(placekick.mb1))

# rearrange columns so that response "good" is last (for later use in bestglm())
placekick.mb <- placekick.mb1[,c(1:11,13,12)]
head(placekick.mb)


######################################################################
# Model selection - Just main effects using other methods.
#    mostly interesting for comparing timing of functions

library(MuMIn)

# Start with full model
# MUST add na.action=
# MUST add na.fail to global model or dredge() will not work.
# Max allowed variables is 30
mod.fit <- glm(good ~ ., family = binomial(link = "logit"), data = placekick.mb,
               na.action = na.fail) 
start.dredge <- Sys.time()
try1 <- dredge(global.model = mod.fit, rank = "AIC")
end.dredge <- Sys.time()
(time.dredge <- end.dredge - start.dredge)
# Get best model 
get.models(object = try1, subset = 1)


######################################################################
# Model selection - Get interactions into model
# 
# Complicated, and no simple way do selection with ALL of the following properties:
#  1. Any main effects from among all variables
#  2. 2-factor interactions only from among those we believe are meaningful
#  3. Models that maintain marginality (variables involved in interactions must 
#     be in the model as main effects)

# Chosen approach: Forward selection, Starting from Chosen Model

# mod.best.lin = best model with linear effects only
# mod.best.upper = all interactions we consider, with main effects produced by "*".
mod.best.lin <- glm(formula = good ~ distance + wind + change + PAT, family = binomial(link = logit), data = placekick.mb) 

mod.best.upper <- glm(formula = good ~ distance*altitude + distance*precip + distance*wind + distance*change + distance*elap30 + distance*PAT +
                        distance*field + distance*temp72 + home*wind + precip*type + precip*field + precip*temp72,
                      family = binomial(link = logit), data = placekick.mb)

step.inter <- step(object = mod.best.lin, scope = list(upper=mod.best.upper), k = 2, trace = FALSE)
summary(step.inter)

# Alternative code to consider possibility that several variables reduce AIC together.
# step() will only consider changes to a model that maintain marginality.
#   Interactions are only added if their main effects are both present
#   Main effects are deleted only if they are not involved in interactions.
# mod.best.lin = best model with linear effects only
# mod.fit.inter = model with all main effects and 
#   plausible interactions
start <- mod.best.lin
maxvars <- mod.best.upper$rank-1
AICs <- extractAIC(mod.best.lin, k=2)
for(i in 1:(maxvars-4)){
  step.1 <- step(object=start, scope=list(upper=mod.best.upper), direction="forward",
                 k = 0, trace=0, steps=1)
  AICs <- rbind(AICs, 
                extractAIC(step.1, k=2))
  start <- step.1
}
head(AICs)
which.min(AICs[,2])

step.inter1 <- step(object = mod.best.lin, scope = list(upper=mod.best.upper), k = 0,
                   steps = which.min(AICs[,2]) - 1)
summary(step.inter1)

### Other approaches to variable selection
# Glmulti on level-2 model using only variables selected in first round
library(glmulti)

#   Leads to same model
search.2.aic <- glmulti(y = mod.best.lin, level = 2, method = "h", crit = "aic",
                        family = binomial(link = "logit"), marginality=TRUE)
weightable(search.2.aic)[1:20,]
print(search.2.aic)
coef(search.2.aic)


######################################################################
# Model selection - Just main effects using other methods.
#    mostly interesting for comparing timing of functions

# glmulti
# The 32- or 64-bit version of R being used needs to have the corresponding 32- or 64-bit version of
#  Java installed on the computer in order for glmulti to work
# Sys.setenv("JAVA_HOME" = "") # If R has problems loading glmulti, may need to use this code
mod.fit.full <- glm(formula = good ~ week + distance + altitude + home + type + precip + wind +
                      change + elap30 + PAT + field, family = binomial(link = logit), data = placekick.mb)
summary(mod.fit.full)

# All subsets regression on all variables with no interactions using AIC
#  glmulti()
start.glmulti <- Sys.time()
search.1.aic <- glmulti(y = mod.fit.full, level = 1, method = "h", crit = "aic",
                        family = binomial(link = "logit"))
end.glmulti <- Sys.time()
(time.glmulti <- end.glmulti - start.glmulti)

slotNames(search.1.aic)
search.1.aic@formulas[[1]]
weightable(search.1.aic)[1:20,]
print(search.1.aic)
coef(search.1.aic)

# bestglm()
library(bestglm)
# Xy= argument, where the value should be a data frame arranged where the last 
#  column in the response variable and all other variables are used as explanatory.
start.bestglm <- Sys.time()
search.bestglm <- bestglm(Xy = placekick.mb, family = binomial, IC = "AIC")
end.bestglm <- Sys.time()
(time.bestglm <- end.bestglm - start.bestglm)
names(search.bestglm)
search.bestglm$BestModels

  
#########################################################
 # Begin diagnostics
 # Convert to EVP form for chosen model
 w <- aggregate(x = good ~ distance + wind + change + PAT, data = placekick.mb, FUN = sum)
 n <- aggregate(x = good ~ distance + wind + change + PAT, data = placekick.mb, FUN = length)
 w.n <- data.frame(w, trials = n$good, prop = round(w$good/n$good,2))
 head(w.n)
 nrow(w.n)  # Number of EVPs
 sum(w.n$trials)  # Number of observations

 # Verify model fit to EVP data matches the model fit to the binary response data format
 mod.prelim1 <- glm(formula = good/trials ~ distance + wind + change + PAT + distance:wind + distance:PAT,
  family = binomial(link = logit), data = w.n, weights = trials)
 round(summary(mod.prelim1)$coefficients, digits = 4)
 round(summary(step.inter)$coefficients, digits = 4)


 # Standardized residuals vs. distance
 dev.new(width = 7, height = 6, pointsize = 12)
 # pdf(file = "c:\\figures\\Figure5.11color.pdf", width = 7, height = 6, colormodel = "cmyk")   # Create plot for book
 stand.resid <- rstandard(model = mod.prelim1, type = "pearson")
 plot(x = w.n$distance, y = stand.resid, ylim = c(min(-3, stand.resid),
  max(3, stand.resid)), ylab = "Standardized Pearson residuals", xlab = "Distance")
 abline(h = c(3, 2, 0, -2, -3), lty = "dotted", col = "blue")
 ord.dist <- order(w.n$distance)
 smooth.stand <- loess(formula = stand.resid ~ distance, data = w.n, weights = trials)
 lines(x = w.n$distance[ord.dist], y = predict(smooth.stand)[ord.dist], lty = "solid", col = "red")
#  dev.off()  # Create plot for book

 # Black-and-white version of plot
 # pdf(file = "c:\\figures\\Figure5.11BW.pdf", width = 7, height = 6, colormodel = "cmyk")   # Create plot for book
 stand.resid <- rstandard(model = mod.prelim1, type = "pearson")
 plot(x = w.n$distance, y = stand.resid, ylim = c(min(-3, stand.resid),
  max(3, stand.resid)), ylab = "Standardized Pearson residuals", xlab = "Distance")
 abline(h = c(3, 2, 0, -2, -3), lty = "dotted", col = "black")
 ord.dist <- order(w.n$distance)
 smooth.stand <- loess(formula = stand.resid ~ distance, data = w.n, weights = trials)
 lines(x = w.n$distance[ord.dist], y = predict(smooth.stand)[ord.dist], lty = "solid", col = "black")
 # dev.off()  # Create plot for book
 
 mod.prelim1q <- glm(formula = good ~ distance + I(distance^2) + wind + change + PAT + distance:wind + distance:PAT,
                    family = binomial(link = logit), data = placekick.mb)
 summary(mod.prelim1q)


##############################################################################
# Diagnostics - preliminary model

 # AIC and other information criteria measures are different than before because we now use the EVP
 #  form of the data (there are a smaller number of rows to the data set)
 AIC(object = mod.prelim1)
 
 # Read in file containing examine.logistic.reg() and run function
 source(file = "Examine.logistic.reg.R")

 # Used with examine.logistic.reg() to rescale numerical values
 one.fourth.root <- function(x) {
  x^0.25
 }
 one.fourth.root(16)  # Example
 # Locator does not currently work in RStudio graphics window.  Open new device first.
 save.info1 <- examine.logistic.reg(mod.fit.obj = mod.prelim1, identify.points = TRUE, scale.n = one.fourth.root,
  scale.cookd = sqrt)
 # save.info1 <- examine.logistic.reg(mod.fit.obj = mod.prelim1, identify.points = FALSE, scale.n = one.fourth.root,
 #  scale.cookd = sqrt) # No identification of points
 # Deviance residual version of plots
 # save.info1 <- examine.logistic.reg(mod.fit.obj = mod.prelim1, identify.points = TRUE,
 #  scale.n = one.fourth.root, scale.cookd = sqrt, pearson.dev = "deviance")
 names(save.info1)

 source("C:\\data\\AllGOFTests.R")  
 
 HL <- HLTest(obj = mod.prelim1, g = 10)
 cbind(HL$observed, round(HL$expect, digits = 1))
 HL
 HL.8 <- HLTest(obj = mod.prelim1, g = 8)
 cbind(HL.8$observed, round(HL.8$expect, digits = 1))
 HL.8
 HL.12 <- HLTest(obj = mod.prelim1, g = 12)
 cbind(HL.12$observed, round(HL.12$expect, digits = 1))
 HL.12
 o.r.test(obj = mod.prelim1)
 stukel.test(obj = mod.prelim1)
 # Surprisingly, significant p-value - p-value is larger for chosen model later
 options(width=60)
 # Examine individual EVPs more closely
 w.n.diag1 <- data.frame(w.n, pi.hat = round(save.info1$pi.hat, 2),
  std.res = round(save.info1$stand.resid, 2),  tail.prob = round(save.info1$tail.prob, 3),
  cookd = round(save.info1$cookd, 2), h = round(save.info1$h,2))
 # w.n.diag1  # Excluded to save space in the book
 
 # Potential EVPs to examine further
 predict(mod.prelim1, newdata=w.n[12:13,], type="response")
 
 # Investigate non-20 yard placekicks further
 mod.prelim1.wo119.120 <- glm(formula = good/trials ~ distance + wind + change + PAT + distance:wind + distance:PAT,
  family = binomial(link = logit), data = w.n[-c(119, 120),], weights = trials)
 round(summary(mod.prelim1.wo119.120)$coefficients, digits = 4)
 # All non-20 yard PATs
 w.n[w.n$distance != 20 & w.n$PAT == 1,]
 p <- length(mod.prelim1$coefficients)
 ck.out <- abs(w.n.diag1$std.res) > 2 | w.n.diag1$cookd > 4/nrow(w.n) | w.n.diag1$h > 3*p/nrow(w.n) | w.n.diag1$tail.prob < 0.025
 # Extract EVPs
 extract.EVPs <- w.n.diag1[ck.out,]
 # Order by distance
 extract.EVPs[order(extract.EVPs$distance),]



###################################################################### 
################################################################################
# 
# Final analysis: Removed all non-20-yard PATs

 placekick.mb2 <- placekick.mb[!(placekick.mb$distance!=20 & placekick.mb$PAT == 1),] 
 nrow(placekick.mb2)  # Number of observations after 4 were removed

 
 mod.fit.full2 <- glm(formula = good ~ week + distance + altitude + home + type + precip + wind +
                       change + elap30 + PAT + field, family = binomial(link = logit), data = placekick.mb2, 
                      na.action = na.fail)
 summary(mod.fit.full2)
 
 start.dredge <- Sys.time()
 try1 <- dredge(global.model = mod.fit.full2, rank = "AIC")
 end.dredge <- Sys.time()
 (time.dredge <- end.dredge - start.dredge)
 # Get best model 
 get.models(object = try1, subset = 1)
 
 mod.best.lin2 <- glm(formula = good ~ distance + wind + change + PAT, family = binomial(link = logit), data = placekick.mb2) 
 # Can no longer estimate distance:PAT 
 inter2 <- c("distance:altitude", "distance:precip", "distance:wind", "distance:change",
            "distance:elap30",  "distance:field", "distance:temp72",
            "home:wind",     "type:precip",   "precip:field", "precip:temp72")
 fmla.inter2 = as.formula(paste("good ~", paste(names(placekick.mb)[-13], collapse=" + "), " + ",
                               paste(inter2, collapse=" + ")))
 mod.fit.inter2 <- glm(formula =  fmla.inter2, family = binomial(link = logit), data = placekick.mb2)
 summary(mod.fit.inter2)
 
 # Forward selection, Starting from Chosen Model
 # step() will only consider changes to a model that maintain marginality.
 #   Interactions are only added if their main effects are both present
 #   Main effects are deleted only if they are not involved in interactions.
 # mod.best.lin = best model with linear effects only
 # mod.fit.inter = model with all main effects and 
 #   plausible interactions
 start2 <- mod.best.lin2
 maxvars2 <- mod.fit.inter2$rank-1
 AICs2 <- extractAIC(mod.best.lin2, k=2)
 for(i in 1:(maxvars2-4)){
   step.2 <- step(object=start2, scope=list(upper=mod.fit.inter2), direction="forward",
                  k = 0, trace=0, steps=1)
   AICs2 <- rbind(AICs2, 
                 extractAIC(step.2, k=2))
   start2 <- step.2
 }
 head(AICs2)
 which.min(AICs2[,2])
 
 step.inter2 <- step(object=mod.best.lin2, scope=list(upper=mod.fit.inter2), k = 0, 
                     steps=which.min(AICs2[,2])-1)
 
 summary(step.inter2)
 
 # EVP form
 w2 <- aggregate(x = good ~ distance + wind + change + PAT, data = placekick.mb2, FUN = sum)
 n2 <- aggregate(x = good ~ distance + wind + change + PAT, data = placekick.mb2, FUN = length)
 w.n2 <- data.frame(w2, trials = n2$good, prop = round(w2$good/n2$good, 2))
 head(w.n2)
 nrow(w.n2)  # Number of EVPs
 sum(w.n2$trials)  # Number of observations
 
 # Verify model fit to EVP data matches the model fit to the binary response data format
 mod.prelim2 <- glm(formula = good/trials ~ distance + wind + change + PAT + distance:wind,
                    family = binomial(link = logit), data = w.n2, weights = trials)
 summary(mod.prelim2)
 
 # Standardized residuals vs. distance
 dev.new(width = 7, height = 6, pointsize = 12)
 stand.resid2 <- rstandard(model = mod.prelim2, type = "pearson")
 # ord.dist <- order(w.n2$distance)
 plot(x = w.n2$distance, y = stand.resid2, ylim = c(min(-3, stand.resid2),
                                                    max(3, stand.resid2)), ylab = "Standardized Pearson residuals", xlab = "Distance")
 abline(h = c(3, 2, 0, -2, -3), lty = "dotted", col = "blue")
 ord.dist2 <- order(w.n2$distance)
 smooth.stand2 <- loess(formula = stand.resid2 ~ distance, data = w.n2, weights = trials)
 lines(x = w.n2$distance[ord.dist2], y = predict(smooth.stand2)[ord.dist2], lty = "solid", col = "red")
 
 
 source("C:\\data\\AllGOFTests.R")  
 
 HL <- HLTest(obj = mod.prelim2, g = 10)
 cbind(HL$observed, round(HL$expect, digits = 1), round(HL$pear.resid, digits=1))
 HL
 HL8 <- HLTest(obj = mod.prelim2, g = 8)
 cbind(HL8$observed, round(HL8$expect, digits = 1))
 HL8
 HL12 <- HLTest(obj = mod.prelim2, g = 12)
 cbind(HL12$observed, round(HL12$expect, digits = 1))
 HL12
 o.r.test(obj = mod.prelim2)
 stukel.test(obj = mod.prelim2)
 
 ################################################################################
 # Diagnostics using data without the non-20 yard placekicks
 source(file = "Examine.logistic.reg.R")
 
 # Used with examine.logistic.reg() for rescaling numerical values
 one.fourth.root <- function(x) {
   x^0.25
 }
 
 save.info2 <- examine.logistic.reg(mod.fit.obj = mod.prelim2, identify.points = TRUE, scale.n = one.fourth.root,
                                    scale.cookd = sqrt)
 
 # Examine individual EVPs more closely
 w.n.diag2 <- data.frame(w.n2, pi.hat = round(save.info2$pi.hat, 2),
                         std.res = round(save.info2$stand.resid, 2), tail.prob = round(save.info2$tail.prob, 3),
                         cookd = round(save.info2$cookd, 2), h = round(save.info2$h, 2)
 )
 # w.n.diag2  # Excluded to save space in the book
 
 # Potential EVPs to examine further
 p <- length(mod.prelim2$coefficients)
 ck.out <- abs(w.n.diag2$std.res) > 2 | w.n.diag2$cookd > 4/nrow(w.n2) | w.n.diag2$h > 3*p/nrow(w.n2) | w.n.diag2$tail.prob < 0.025
 extract.EVPs2 <- w.n.diag2[ck.out,]  # Extract EVPs
 extract.EVPs2[order(extract.EVPs2$distance),]  # Order by distance
 
 
 ################################################################################
 # Model interpretation
 
 # OR estimates
 library(package = mcprofile)
 
 OR.name <- c("Change", "PAT", "Distance, 10-yard decrease, windy", "Distance, 10-yard decrease, not windy",
              "Wind, distance = 20", "Wind, distance = 30", "Wind, distance = 40", "Wind, distance = 50",
              "Wind, distance = 60")
 var.name <- c("int", "distance", "wind", "change", "PAT", "distance:wind")
 
 K <- matrix(data = c(0,  0, 0, 1, 0,  0,
                      0,  0, 0, 0, 1,  0,
                      0, -10, 0, 0, 0, -10,
                      0, -10, 0, 0, 0,  0,
                      0,  0, 1, 0, 0, 20,
                      0,  0, 1, 0, 0, 30,
                      0,  0, 1, 0, 0, 40,
                      0,  0, 1, 0, 0, 50,
                      0,  0, 1, 0, 0, 60),
             nrow = 9, ncol = 6, byrow = TRUE, dimnames = list(OR.name, var.name))
 # K # Check matrix - excluded to save space
 linear.combo <- mcprofile(object = mod.prelim2, CM = K)
 ci.log.OR <- confint(object = linear.combo, level = 0.99, adjust = "none")
 # ci.log.OR
 exp(ci.log.OR)
 
 # Wald CIs (if desired)
 save.wald <- wald(linear.combo)
 save.wald
 ci.log.OR.wald <- confint(object = save.wald, level = 0.99, adjust = "none")
 exp(ci.log.OR.wald)
 
 
 # Examine probability of success for PATs vs. field goals
 predict(object = mod.prelim2, newdata = data.frame(distance = c(20, 20), wind = c(0, 0),
                                                    change = c(0, 0), PAT = c(1, 0)), type = "response")
 
 # Using mcprofile to obtain profile LR
 K <- matrix(data = c(1, 20, 0, 0, 1, 0,
                      1, 20, 0, 0, 0, 0),
             nrow = 2, ncol = 6, byrow = TRUE, dimnames = list(c("PAT", "FG"), var.name))
 # K # Check matrix - excluded to save space
 linear.combo <- mcprofile(object = mod.prelim2, CM = K)
 ci.lin.pred <- confint(object = linear.combo, level = 0.99, adjust = "none")
 ci.lin.pred
 # exp(ci.lin.pred$estimate)/(1 + exp(ci.lin.pred$estimate))
 # plogis(q = c(4.14, 2.88))
 # as.matrix() is needed to get the proper class for plogis()
 #   because ci.lin.pred$estimate is a data frame
 round(plogis(q = as.matrix(ci.lin.pred$estimate)), digits = 3)  # as.numeric() and as.vector() do not work
 round(plogis(q = as.matrix(ci.lin.pred$confint)), digits = 3)
 
 #Elliott's kick discussed in Bilder and Loughin (1998)
 K <- matrix(data = c(1, 42, 0, 1, 0, 0), nrow = 1, ncol = 6, byrow = TRUE)
 K
 linear.combo <- mcprofile(object = mod.prelim2, CM = K)
 ci.lin.pred <- confint(object = linear.combo, level = 0.99, adjust = "none")
 round(plogis(q = as.matrix(ci.lin.pred$estimate)), digits = 3)
 round(plogis(q = as.matrix(ci.lin.pred$confint)), digits = 3)
 
 # Plot - Probability of success for four combinations of explanatory variables
 beta.hat <- mod.prelim2$coefficients
 # Change = 0, wind = 0
 dev.new(width = 7, height = 6, pointsize = 12)
 # pdf(file = "c:\\figures\\Figure5.13color.pdf", width = 7, height = 6, colormodel = "cmyk")   # Create plot for book
 curve(expr = plogis(beta.hat[1] + beta.hat[2]*x), lty = "solid", xlim = c(18, 66), ylim = c(0, 1), lwd = 2,
       col = "red", panel.first = grid(col = "gray", lty = "dotted"), ylab = "Estimated probability of success",
       xlab = "Distance")
 # change = 1, wind = 0
 curve(expr = plogis(beta.hat[1] + beta.hat[2]*x + beta.hat[4]), lty = "dashed",
       lwd = 2 , col = "darkgreen", add = TRUE)
 # change = 0, wind = 1
 curve(expr = plogis(beta.hat[1] + beta.hat[2]*x + beta.hat[3] + beta.hat[6]*x), lty = "dotted",
       lwd = 2, col = "blue", add = TRUE)
 # change = 1, wind = 1
 curve(expr = plogis(beta.hat[1] + beta.hat[2]*x + beta.hat[3] + beta.hat[4] + beta.hat[6]*x), lty = "dotdash",
       lwd = 2, col = "purple", add = TRUE)
 names1 <- c("Change = 0, Wind = 0", "Change = 1, Wind = 0", "Change = 0, Wind = 1", "Change = 1, Wind = 1")
 legend(x = 20, y = 0.39, legend = names1, lty = c("solid", "dashed", "dotted", "dotdash"),
        col = c("red","darkgreen","blue","purple"), bty = "n", cex = 1, lwd = 2)
 # dev.off()  # Create plot for book
 
 # Black-and-white version of plot
 # pdf(file = "c:\\figures\\Figure5.13BW.pdf", width = 7, height = 6, colormodel = "cmyk")   # Create plot for book
 curve(expr = plogis(beta.hat[1] + beta.hat[2]*x), lty = "solid", xlim = c(18, 66), ylim = c(0, 1), lwd = 2,
       col = "black", ylab = "Estimated probability of success", xlab = "Distance")
 curve(expr = plogis(beta.hat[1] + beta.hat[2]*x + beta.hat[4]), lty = "dashed",
       lwd = 2 , col = "black", add = TRUE)
 curve(expr = plogis(beta.hat[1] + beta.hat[2]*x + beta.hat[3] + beta.hat[6]*x), lty = "dotted",
       lwd = 2, col = "black", add = TRUE)
 curve(expr = plogis(beta.hat[1] + beta.hat[2]*x + beta.hat[3] + beta.hat[4] + beta.hat[6]*x), lty = "dotdash",
       lwd = 2, col = "black", add = TRUE)
 names1 <- c("Change = 0, Wind = 0", "Change = 1, Wind = 0", "Change = 0, Wind = 1", "Change = 1, Wind = 1")
 legend(x = 20, y = 0.39, legend = names1, lty = c("solid", "dashed", "dotted", "dotdash"),
        col = c("black","black","black","black"), bty = "n", cex = 1, lwd = 2)
 # dev.off()  # Create plot for book
 
 
 # Plot - Probability of success for two combinations of explanatory variables with CIs
 # Most of this function is from Chapter 2
 ci.pi <- function(newdata, mod.fit.obj, alpha){
   linear.pred <- predict(object = mod.fit.obj, newdata = newdata, type = "link", se = TRUE)
   CI.lin.pred.lower <- linear.pred$fit - qnorm(p = 1-alpha/2)*linear.pred$se
   CI.lin.pred.upper <- linear.pred$fit + qnorm(p = 1-alpha/2)*linear.pred$se
   CI.pi.lower <- exp(CI.lin.pred.lower) / (1 + exp(CI.lin.pred.lower))
   CI.pi.upper <- exp(CI.lin.pred.upper) / (1 + exp(CI.lin.pred.upper))
   list(pi.hat = plogis(linear.pred$fit), lower = CI.pi.lower, upper = CI.pi.upper)
 }
 # Test
 ci.pi(newdata = data.frame(distance = c(20, 20), wind = c(0, 0),
                            change = c(0, 0), PAT = c(1, 0)), mod.fit.obj = mod.prelim2, alpha = 0.10)
 
 # Change = 0, wind = 0
 dev.new(width = 7, height = 6, pointsize = 12)
 # pdf(file = "c:\\figures\\Figure5.14color.pdf", width = 7, height = 6, colormodel = "cmyk")   # Create plot for book
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 0, change = 0, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$pi.hat,
       xlim = c(18, 66), lty = "solid", lwd = 2, col = "red", xlab = "Distance", ylab = "Estimated probability of success",
       ylim = c(0, 1), panel.first = grid(col = "gray", lty = "dotted"))
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 0, change = 0, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$lower,
       lty = "dotted", lwd = 2, col = "red", add = TRUE)
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 0, change = 0, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$upper,
       lty = "dotted", lwd = 2, col = "red", add = TRUE)
 
 # Change = 1 and wind = 1
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 1, change = 1, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$pi.hat,
       lty = "dotdash", lwd = 2, col = "purple", add = TRUE)
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 1, change = 1, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$lower,
       lty = "dotted", lwd = 2, col = "purple", add = TRUE)
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 1, change = 1, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$upper,
       lty = "dotted", lwd = 2, col = "purple", add = TRUE)
 
 names1 <- c("Estimated Probability", "99% Confidence Interval")
 text(x = 22, y = 0.38, "Least risky")
 legend(x = 17, y = 0.38, legend = names1, lty = c("solid", "dotted", "dotted"), col = c("red","red"), bty = "n", lwd = 2)
 text(x = 21.5, y = 0.18, "Most risky")
 legend(x = 17, y = 0.18, legend = names1, lty = c("dotdash", "dotted", "dotted"), col = c("purple","purple"), bty = "n", lwd = 2)
 # dev.off()  # Create plot for book
 
 
 # Black-and-white version of plot
 # pdf(file = "c:\\figures\\Figure5.14BW.pdf", width = 7, height = 6, colormodel = "cmyk")   # Create plot for book
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 0, change = 0, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$pi.hat,
       xlim = c(18, 66), lty = "solid", lwd = 2, col = "black", xlab = "Distance", ylab = "Estimated probability of success",
       ylim = c(0, 1))
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 0, change = 0, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$lower,
       lty = "dotted", lwd = 2, col = "black", add = TRUE)
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 0, change = 0, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$upper,
       lty = "dotted", lwd = 2, col = "black", add = TRUE)
 
 # Change = 1 and wind = 1
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 1, change = 1, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$pi.hat,
       lty = "dotdash", lwd = 2, col = "black", add = TRUE)
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 1, change = 1, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$lower,
       lty = "dotted", lwd = 2, col = "black", add = TRUE)
 curve(expr = ci.pi(newdata = data.frame(distance = x, wind = 1, change = 1, PAT = 0), mod.fit.obj = mod.prelim2, alpha = 0.10)$upper,
       lty = "dotted", lwd = 2, col = "black", add = TRUE)
 
 names1 <- c("Estimated Probability", "99% Confidence Interval")
 text(x = 22, y = 0.38, "Least risky")
 legend(x = 17, y = 0.38, legend = names1, lty = c("solid", "dotted", "dotted"), col = c("black","black"), bty = "n", lwd = 2)
 text(x = 21.5, y = 0.18, "Most risky")
 legend(x = 17, y = 0.18, legend = names1, lty = c("dotdash", "dotted", "dotted"), col = c("black","black"), bty = "n", lwd = 2)
 # dev.off()  # Create plot for book
 
 