# Example: Placekicking, inference after variable selection

# Original data set
set1 <- read.csv(file = "c:\\data\\Placekick2.csv")
head(set1, n = 3)


################################################################################
# Split data

  n <- nrow(set1)
  # Select random half of numbers between 1 and n
  set.seed(31010742)  # Obtained number by using runif() once
  index.sample <- sample.int(n = n, size = round(n/2))
  head(sort(index.sample))
  head(index.sample)

  # Form the two data sets
  set.train <- set1[-index.sample,]
  set.valid <- set1[index.sample,]
  nrow(set.train)
  nrow(set.valid)
  n

  options(width = 60)
  head(set.train, n = 3)
  head(set.valid, n = 3)

  tail(set.train, n = 3)
  tail(set.valid, n = 3)
  options(width = 80)

  # Examine the Denver placekicks more closely
  xtabs(formula = ~ good + distance, data = set.train[set.train$city == "DEN",])
  xtabs(formula = ~ good + distance, data = set.valid[set.valid$city == "DEN",])


###########################################################################################
# Choose terms to include in the model

  # Choose main effects using other methods, city is excluded
  library(MuMIn)
  mod.fit <- glm(formula = good ~ . - city, family = binomial(link = "logit"), data = set.train,
    na.action = na.fail)
  summary(mod.fit)
  allsub.aic <- dredge(global.model = mod.fit, rank = "AIC")
  allsub.aic[allsub.aic$delta < 1,]

  # Model chosen
  mod.best.lin <- glm(formula = good ~ distance + altitude, family = binomial(link = logit),
    data = set.train)
  summary(mod.best.lin)
  AIC(mod.best.lin)

  # mod.fit.inter = model with all main effects and plausible interactions
  main.effects <- "week + distance + altitude + home + type + precip + wind + change + elap30 + PAT + field + temp72"    # 12
  interactions <- "distance:altitude + distance:precip + distance:wind + distance:change + distance:elap30 + distance:PAT + distance:field + distance:temp72 + home:wind + type:precip + precip:field + precip:temp72"  #12
  complete.formula <- paste("good ~", main.effects, "+", interactions)
  mod.fit.inter <- glm(formula = formula(complete.formula), family = binomial(link = logit), data = set.train)
  summary(mod.fit.inter)

  # Number of potential terms in model
  maxvars <- 24
  # Save results from for loop
  save.res <- matrix(data = NA, nrow = maxvars - 4, ncol = 2)
  # Smallest model
  start.model <- mod.best.lin
  counter <- 1
  for(i in 1:(maxvars-4)){
    step.1 <- step(object = start.model, scope = list(upper = mod.fit.inter), direction = "forward", k = 0, trace = 0, steps = 1)
    save.res[counter, ] <- c(round(AIC(step.1),2), step.1$anova$Step[2])
    start.model <- step.1
    counter <- counter + 1
  }
  save.res

  # A different approach
  step(object = mod.best.lin, scope = list(upper = mod.fit.inter), direction = "forward", k = 2, trace = 1)


###########################################################################################
# EVP form and evaluate model

  w <- aggregate(x = good ~ distance + altitude, data = set.train, FUN = sum)
  n <- aggregate(x = good ~ distance + altitude, data = set.train, FUN = length)
  w.n <- data.frame(w, trials = n$good, prop = round(w$good/n$good,2))
  head(w.n)
  nrow(w.n)  # 417 EVPs
  sum(w.n$trials)  # 1101 observations
  table(w.n$trials)  # 304 EVPs have one trial, 68 EVPs have two trials, 13 EVPs have 3 trials
  # EVP form has many cases where a normal or chi-square distribution approximation
  #   will not work well

  mod.prelim1 <- glm(formula = good/trials ~ distance + altitude,
    family = binomial(link = logit), data = w.n, weights = trials)
  summary(mod.prelim1)

  library(car)
  Anova(mod.prelim1)

  # Used with examine.logistic.reg() for rescaling numerical values
  one.fourth.root <- function(x) {
    x^0.25
  }

  # Obtain function from Examine.logistic.reg.R
  save.info1 <- examine.logistic.reg(mod.fit.obj = mod.prelim1, identify.points = TRUE, scale.n = one.fourth.root,
    scale.cookd = sqrt)
  # deviance/df = 0.88


  w.n.diag1 <- data.frame(w.n, pi.hat = round(save.info1$pi.hat, 2), std.res = round(save.info1$stand.resid, 2),
                         tail.prob = round(save.info1$tail.prob, 3), cookd = round(save.info1$cookd, 2), h = round(save.info1$h,2))
  # Potential EVPs to examine further
  p <- length(mod.prelim1$coefficients)
  ck.out <- abs(w.n.diag1$std.res) > 2 | w.n.diag1$cookd > 4/nrow(w.n) | w.n.diag1$h > 3*p/nrow(w.n) | w.n.diag1$tail.prob < 0.025
  extract.EVPs <- w.n.diag1[ck.out,]  # Extract EVPs
  extract.EVPs[order(extract.EVPs$distance),]  # Order by distance
  extract.EVPs[order(extract.EVPs$tail.prob),]  # Order by distance
  # Tail probabilities (smallest are 0.005, 0.031, 0.034) are reasonable given the sample size
  # Observation #12 has the smallest tail probability - it has 1 success out of 4 trials
  #    at a distance of 38 yards and an altitude of 0. Its pihat is 0.89.
  #    This is sort like the 18 yard field goals issue for the original placekicking data set

  mod.fit.train <- glm(formula = good ~ distance + altitude, family = binomial(link = logit), data = set.train)
  summary(mod.fit.train)
  Anova(mod.fit.train)
  confint(mod.fit.train)
  

###############################################################################
# Validation data

  mod.fit.valid <- glm(formula = good ~ distance + altitude, family = binomial(link = logit), data = set.valid)
  summary(mod.fit.valid)
  Anova(mod.fit.valid)
  confint(mod.fit.valid)

  data.frame(train = round(mod.fit.train$coefficients, 6), valid = round(mod.fit.valid$coefficients, 6))



###############################################################################
# Number of successes and failures in the data

  table(set.train$good)
  table(set.valid$good)

  set.train$altitude1000 <- set.train$altitude > 1000
  set.valid$altitude1000 <- set.valid$altitude > 1000
  table(set.train$good, set.train$altitude1000)
  table(set.valid$good, set.valid$altitude1000)










#