# Example: Inference after variable selection


# Simulation to demonstrate problem with performing hypothesis tests after
#   variable selection.
# 1. Generating n=100 observations with p=20 independent normally distributed 
#      variables and a Po(10) response.
# 2. The response is not related to the explanatory variables, so the null 
#      hypothesis of a zero regression parameter is true in every case.
# 3. Then performing all subsets selection with BIC (which should report 
#      the fewest variables among all our criteria)
# 4. When variables are selected, run a glm() on the selected variables and 
#      test the significance of their parameters
# 5. Store the p-values in test.p and summarize them

n <- 100
p <- 20
reps <- 20
# Generate data
set.seed(393822334)
X.all <- matrix(rnorm(n=n*p*reps), ncol=p)
Y.all <- rpois(n=n*reps, lambda=10)
rep.num <- rep(1:reps, each=n)

test.p <- matrix(NA, nrow=reps, ncol=p)
# Run analysis loop.  May take a minute or so.
library(BMA)
for(i in 1:reps){
  y1 <- Y.all[rep.num==i]
  x1 <- X.all[rep.num==i,]
  allsub1 <- bic.glm(x=x1, y=y1, glm.family=poisson(link="log"))
  vars <- allsub1$which[1,]
  chosen <- sum(vars)
  if(chosen>0){
    mod1 <- glm(y1~., data=data.frame(y1,x1[,vars]), family=poisson(link="log"))
    p.chosen <- summary(mod1)$coefficients[-1,4]
  }
  test.p[i,vars] <- p.chosen 
}

# Number of variables selected by BIC in each data set 
reps - apply(X=is.na(test.p), MARGIN=1, FUN=sum)
# Summary of all p-values  
summary(as.vector(test.p))
# Number of times a test was done (out of p*reps opportunities)
length(na.omit(as.vector(test.p)))
# Proportion of tests with p<0.05
mean(na.omit(as.vector(test.p))<0.05)

#
###############################################################
# What if we had had more data?  
# Suppose that we originally had more observations and we used 1/2 
#   for the variable selection as above, with the remaining 1/2 
#   reserved for tests and confidence intervals.
# Since all of our data are randomly generated, we can simulate this 
#   just by generating n more observations using the same model as 
#   above. Then we rerun the tests on these new data.
# We do this for each of the data sets we used above.  

# Generate new validation data
X.all2 <- matrix(rnorm(n=n*p*reps), ncol=p)
Y.all2 <- rpois(n=n*reps, lambda=10)
rep.num2 <- rep(1:reps, each=n)
vars2 <- !is.na(test.p)

test.p2 <- matrix(NA, nrow=reps, ncol=p)
# Rerun tests using analysis data
library(BMA)
for(i in 1:reps){
  y1 <- Y.all2[rep.num2==i]
  x1 <- X.all2[rep.num2==i,]
  vars <- vars2[i,]
  chosen <- sum(vars)
  if(chosen>0){
    mod1 <- glm(y1~., data=data.frame(y1,x1[,vars]), family=poisson(link="log"))
    p.chosen <- summary(mod1)$coefficients[-1,4]
  }
  test.p2[i,vars] <- p.chosen 
}

# Summary of all p-values  
summary(as.vector(test.p2))
# Number of times a test was done (out of p*reps opportunities)
length(na.omit(as.vector(test.p2)))
# Proportion of tests with p<0.05
mean(na.omit(as.vector(test.p2))<0.05)

