# Example: Placekicking using splines

  placekick <- read.csv(file = "C:\\data\\Placekick.csv", header = TRUE, sep = ",")
  head(placekick)
  tail(placekick)



#####################################################################
# Distance polynomials

  mod.fit1 <- glm(formula = good ~ distance, family = binomial(link = logit), data = placekick)
  # summary(mod.fit1)
  mod.fit2 <- glm(formula = good ~ distance + I(distance^2), family = binomial(link = logit), data = placekick)
  # summary(mod.fit2)
  mod.fit3 <- glm(formula = good ~ distance + I(distance^2) + I(distance^3), family = binomial(link = logit), data = placekick)
  # summary(mod.fit3)


  library(package = car)
  Anova(mod.fit3) # Marginally significant terms
  data.frame(model1 = AIC(mod.fit1), model2 = AIC(mod.fit2), model3 = AIC(mod.fit3))
  data.frame(model1 = BIC(mod.fit1), model2 = BIC(mod.fit2), model3 = BIC(mod.fit3))
  # Note: While the AIC values here are lower for the 2nd order and 3rd order models,
  #   other explanatory variables are not included with distance. These other variables
  #   are shown to be important elsewhere in the book.


#####################################################################
# Truncated power terms

  # This function creates the truncated power terms
  tp <- function(x, knots) {
    numb.knots <- length(knots)
    numb.x <- length(x)
    tp.mat <- matrix(data = NA, nrow = numb.x, ncol = numb.knots)
    for (d in 1:numb.knots) {
      tp.mat[1:numb.x,d] <- ifelse(test = x > knots[d], yes = (x - knots[d])^3, no = 0)
    }
    #colnames(tp.mat) <- paste("tp", 1:numb.knots, sep = "")   # If knots are not nice numerical quantities, use this instead
    colnames(tp.mat) <- paste("tp", knots, sep = "")
    as.data.frame(tp.mat)
  }

  knots4 <- seq(from = 25, to = 55, by = 10)

  # Test
  # tp(x = 30:31, knots = knots4)

  placekick2 <- cbind(placekick, tp(x = placekick$distance, knots = knots4))
  head(placekick2[, c("distance", "tp25", "tp35", "tp45", "tp55")])
  tail(placekick2[, c("distance", "tp25", "tp35", "tp45", "tp55")])

  # Estimate model with truncated power terms
  mod.fit.tp <- glm(formula = good ~ distance + I(distance^2) + I(distance^3) +
    tp25 + tp35 + tp45 + tp55, family = binomial(link = logit),
    data = placekick2)
  summary(object = mod.fit.tp)
  anova(mod.fit3, mod.fit.tp, test = "Chisq")
  AIC(mod.fit.tp)
  AIC(mod.fit3)


#####################################################################
# B-spline and cubic spline

  library(package = splines)

  options(width = 60) # Need due to wide output produced below (does not fit well into book)

  mod.fit.bs <- glm(formula = good ~ bs(x = distance, knots = knots4), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.bs)
  Anova(mod.fit.bs)

  mod.fit.ns <- glm(formula = good ~ ns(x = distance, knots = knots4), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns)

  data.frame(model.bs = AIC(mod.fit.bs), model.ns = AIC(mod.fit.ns))
  data.frame(model.bs = BIC(mod.fit.bs), model.ns = BIC(mod.fit.ns))

  options(width = 80)


  # Estimate odds ratios with natural splines
  prob <- predict(object = mod.fit.ns, newdata = data.frame(distance = c(20,30)),
    type = "response")
  prob
  prob[1]/(1-prob[1]) / (prob[2]/(1-prob[2]))
  link <- predict(object = mod.fit.ns, newdata = data.frame(distance = c(20,30)),
    type = "link")
  link
  exp(link[1] - link[2])


  ###################
  # To find a CI for the odds ratio using mcprofile, we need to determine the values of the
  #   basis functions for the distances of interest. Once we have this information,
  #   the use of mcprofile follows as in the logistic regression chapter. Below
  #   is how this can be done for an odds ratio comparing 20 to 30-yard placekicks.
  library(package = mcprofile)
  distance.basis <- ns(x = placekick$distance, knots = knots4)
  func.val <- predict(object = distance.basis, newx = c(20,30))
  #func.val <- predict(object = distance.basis, newx = c(30,40))
  #func.val <- predict(object = distance.basis, newx = c(40,50))
  func.val[1:2,]
  options(width = 60)  # K is too wide for book
  # h(a) - h(b)
  K <- matrix(data = c(1, func.val[1,]) - c(1, func.val[2,]), nrow = 1, ncol = 6,
    byrow = TRUE)
  K
  # Estimated odds ratio
  exp(sum(mod.fit.ns$coefficients * K))
  options(width = 80)
  # mcprofile calculations
  linear.combo.OR <- mcprofile(object = mod.fit.ns, CM = K)
  ci.log.OR <- confint(object = linear.combo.OR, level = 0.95) # Remember to use the adjust argument if more than one odds ratio is calculated
  ci.log.OR
  exp(ci.log.OR)



  library(package = emmeans)
  calc.est <- emmeans(object =  mod.fit.ns, specs = ~ distance, params = "knots4",
    at = list(distance = c(20,30)), type = "response")
  summary(calc.est, level = 0.95)
  confint(object = contrast(object = calc.est, method = "pairwise"), level = 0.95)

  # Example using 3 degrees of freedom spline
  #  mod.fit.ns.df3 <- glm(formula = good ~ ns(x = distance, df = 3), family = binomial(link = logit), data = placekick)
  #  calc.est2 <- emmeans(object =  mod.fit.ns.df3, specs = ~ distance, at = list(distance = c(20,30)), type = "response")
  # summary(calc.est2)
  # confint(object = contrast(object = calc.est2, method = "pairwise"), level = 0.95)


#####################################################################
# Plot

  #Find the observed proportion of successes at each distance
  w <- aggregate(x = good ~ distance, data = placekick, FUN = sum)
  n <- aggregate(x = good ~ distance, data = placekick, FUN = length)
  w.n <- data.frame(distance = w$distance, success = w$good, trials = n$good, proportion = round(w$good/n$good,4))
  head(w.n)
  tail(w.n)

  # Black-and-white version of plot
  # pdf(file = "c:\\figures\\FigureSpline1BW.pdf", width = 7, height = 6, colormodel = "cmyk")   # Create plot for book
  dev.new(width = 7, height = 6, pointsize = 12)
  symbols(x = w$distance, y = w$good/n$good, circles = sqrt(n$good), inches = 0.5,
    xlab="Distance (yards)", ylab="Estimated probability")
  # Use plot() rather than symbols if you do not want to see the bubbles
  #plot(x = w$distance, y = w$good/n$good, circles = sqrt(n$good),
  #  xlab="Distance (yards)", ylab="Estimated probability", type = "n")
  curve(expr = predict(object = mod.fit1, newdata = data.frame(distance = x),
    type = "response"), lty = 1, lwd = 2, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit2, newdata = data.frame(distance = x),
    type = "response"), lty = 1, lwd = 1, add = TRUE,  xlim = c(18,66))
  curve(expr = predict(object = mod.fit3, newdata = data.frame(distance = x),
    type = "response"), lty = 2, lwd = 1, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit.bs, newdata = data.frame(distance = x),
    type = "response"), lty = 3, lwd = 2, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit.ns, newdata = data.frame(distance = x),
    type = "response"), lty = 4, lwd = 2, add = TRUE, xlim = c(18,66)) #dot dot dash
  legend(x = 20, y = 0.4, legend = c("First order", "Second order", "Third order", "B-spline",
    "Natural spline"), lty = c(1,1,2,3,4), lwd = c(2,1,1,2,2), bty = "n")
  # dev.off()  # Create plot for book

  # This is how the truncated power model can be plotted
  # curve(expr = predict(object = mod.fit.tp, newdata = data.frame(distance = x,
  #   tp(x = x, knots = seq(from = 25, to = 55, by = 10))), type = "response"),
  #   col = "black", add = TRUE, xlim = c(18,66))

  # Compare the estimated probabilities
  pred1 <- round(predict(object = mod.fit1, newdata = data.frame(distance = 18:66),
    type = "response"), 4)
  pred2 <- round(predict(object = mod.fit2, newdata = data.frame(distance = 18:66),
    type = "response"), 4)
  pred3 <- round(predict(object = mod.fit3, newdata = data.frame(distance = 18:66),
    type = "response"), 4)
  pred.bs <- round(predict(object = mod.fit.bs, newdata = data.frame(distance = 18:66),
    type = "response"), 4)
  pred.ns <- round(predict(object = mod.fit.ns, newdata = data.frame(distance = 18:66),
    type = "response"), 4)
  pred <- data.frame(distance = 18:66, pred1, pred2, pred3, pred.bs, pred.ns)
  head(pred)

  # How far apart are the estimated probabilities?
  pairwise.diff <- function(x, y, distance) {
    diff <- round(x  - y, 4)
    set1 <- data.frame(distance, x, y, diff)
    set1[order(set1$diff),]
  }

  pairwise.diff(pred1, pred2, distance = 18:66)
  pairwise.diff(pred1, pred3, distance = 18:66)
  pairwise.diff(pred1, pred.ns, distance = 18:66)
  pairwise.diff(pred1, pred.bs, distance = 18:66)


#####################################################################
# Other knot choices

  # Estimate models with specified knots
  knots <- seq(from = 25, to = 55, by = 3)
  mod.fit.ns1 <- glm(formula = good ~ ns(x = distance, knots = knots), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns1)

  knots <- seq(from = 25, to = 55, by = 6)
  mod.fit.ns2 <- glm(formula = good ~ ns(x = distance, knots = knots), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns2)

  knots <- seq(from = 25, to = 55, by = 10)
  mod.fit.ns3 <- glm(formula = good ~ ns(x = distance, knots = knots), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns3)

  knots <- seq(from = 25, to = 55, by = 15)
  mod.fit.ns4 <- glm(formula = good ~ ns(x = distance, knots = knots), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns4)

  knots <- seq(from = 25, to = 55, by = 30)
  mod.fit.ns5 <- glm(formula = good ~ ns(x = distance, knots = knots), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns5)

  # Comparison
  data.frame(by3 = AIC(mod.fit.ns1), by6 = AIC(mod.fit.ns2),
    by10 = AIC(mod.fit.ns3), by15 = AIC(mod.fit.ns4), by30 = AIC(mod.fit.ns5),
    none = AIC(mod.fit3))

  data.frame(by3 = BIC(mod.fit.ns1), by6 = BIC(mod.fit.ns2),
    by10 = BIC(mod.fit.ns3), by15 = BIC(mod.fit.ns4), by30 = BIC(mod.fit.ns5),
    none = BIC(mod.fit3))


  # Estimate models with specified degrees of freedom
  mod.fit.ns.df12 <- glm(formula = good ~ ns(x = distance, df = 12), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns.df12)
  # Some regression parameters cannot be estimated due to the large number of 20-yard placekicks
  #   (some quantiles end up being the same)
  names(mod.fit.ns.df12)
  mod.fit.ns.df12$terms # Shows locations of the knots

  mod.fit.ns.df7 <- glm(formula = good ~ ns(x = distance, df = 7), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns.df7)
  # Signs of non-convergence - very large SEs
  mod.fit.ns.df7$terms

  mod.fit.ns.df5 <- glm(formula = good ~ ns(x = distance, df = 5), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns.df5)
  mod.fit.ns.df5$terms

  mod.fit.ns.df4 <- glm(formula = good ~ ns(x = distance, df = 4), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns.df4)
  mod.fit.ns.df4$terms

  mod.fit.ns.df3 <- glm(formula = good ~ ns(x = distance, df = 3), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns.df3)
  mod.fit.ns.df3$terms

  # Comparison
  data.frame(by3 = AIC(mod.fit.ns1), by6 = AIC(mod.fit.ns2),
    by10 = AIC(mod.fit.ns3), by15 = AIC(mod.fit.ns4), by30 = AIC(mod.fit.ns5),
    df12 = AIC(mod.fit.ns.df12), df7 = AIC(mod.fit.ns.df7), df5 = AIC(mod.fit.ns.df5),
    df4 = AIC(mod.fit.ns.df4), df3 = AIC(mod.fit.ns.df3), none = AIC(mod.fit3))

  data.frame(by3 = BIC(mod.fit.ns1), by6 = BIC(mod.fit.ns2),
    by10 = BIC(mod.fit.ns3), by15 = BIC(mod.fit.ns4), by30 = BIC(mod.fit.ns5),
    df12 = BIC(mod.fit.ns.df12), df7 = BIC(mod.fit.ns.df7), df5 = BIC(mod.fit.ns.df5),
    df4 = BIC(mod.fit.ns.df4), df3 = BIC(mod.fit.ns.df3), none = BIC(mod.fit3))



  # Black-and-white version of plot
  # pdf(file = "c:\\figures\\FigureSplineKnotsBW.pdf", width = 7, height = 6, colormodel = "cmyk")   # Create plot for book
  dev.new(width = 7, height = 6, pointsize = 12)
  symbols(x = w$distance, y = w$good/n$good, circles = sqrt(n$good), inches = 0.5,
    xlab = "Distance (yards)", ylab = "Estimated probability")
  curve(expr = predict(object = mod.fit1, newdata = data.frame(distance = x),
    type = "response"), lty = 1, lwd = 2, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit3, newdata = data.frame(distance = x),
    type = "response"), lty = 2, lwd = 1, add = TRUE, xlim = c(18,66))

  curve(expr = predict(object = mod.fit.ns1, newdata = data.frame(distance = x),
    type = "response"), lty = 4, lwd = 2, add = TRUE, xlim = c(18,66))
  #curve(expr = predict(object = mod.fit.ns2, newdata = data.frame(distance = x),
  #  type = "response"), lty = 1, lwd = 2, add = TRUE, xlim = c(18,66))
  #curve(expr = predict(object = mod.fit.ns3, newdata = data.frame(distance = x),
  #  type = "response"), lty = 1, lwd = 2, add = TRUE, xlim = c(18,66))
  #curve(expr = predict(object = mod.fit.ns4, newdata = data.frame(distance = x),
  #  type = "response"), lty = 1, lwd = 2, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit.ns5, newdata = data.frame(distance = x),
    type = "response"), lty = 3, lwd = 1, add = TRUE, xlim = c(18,66))
  legend(x = 20, y = 0.4, legend = c("First order", "Third order", "Knots 25 to 55 by 3",
    "Knots 25 to 55 by 30"), lty = c(1,2,4,3), lwd = c(2,1,2,1), bty = "n")


  dev.new(width = 7, height = 6, pointsize = 12)
  symbols(x = w$distance, y = w$good/n$good, circles = sqrt(n$good), inches = 0.5,
    xlab = "Distance (yards)", ylab = "Estimated probability")
  curve(expr = predict(object = mod.fit1, newdata = data.frame(distance = x),
    type = "response"), lty = 1, lwd = 2, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit3, newdata = data.frame(distance = x),
    type = "response"), lty = 2, lwd = 1, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit.ns.df3, newdata = data.frame(distance = x),
    type = "response"), lty = 4, lwd = 2, add = TRUE, xlim = c(18,66))
  # curve(expr = predict(object = mod.fit.ns.df4, newdata = data.frame(distance = x),
  #  type = "response"), lty = "131353", lwd = 1, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit.ns.df5, newdata = data.frame(distance = x),
    type = "response"), lty = 3, lwd = 1, add = TRUE, xlim = c(18,66))
  legend(x = 20, y = 0.4, legend = c("First order", "Third order", "df = 3",
     "df = 5"), lty = c(1,2,4,3), lwd = c(2,1,2,1), bty = "n")



#####################################################################
# Smoothing spline

  library(package = mgcv)

  mod.fit.ps <- gam(formula = good ~ s(distance, bs = "cr"), family = binomial(link = "logit"), data = placekick)
  mod.fit.ps
  summary(mod.fit.ps)
  plot(mod.fit.ps)  # plot.gam()
  class(mod.fit.ps)
  plot(mod.fit.ps, residuals = TRUE)
  anova(mod.fit.ps) # Same as givein in the summary() output
  names(mod.fit.ps)
  AIC(mod.fit.ps)
  predict(object = mod.fit.ps, newdata = data.frame(distance = c(20,30)),
    type = "response")

  # pdf(file = "c:\\figures\\FigureSmoothingSpline.pdf", width = 7, height = 6, colormodel = "cmyk")   # Create plot for book
  dev.new(width = 7, height = 6, pointsize = 12)
  symbols(x = w$distance, y = w$good/n$good, circles = sqrt(n$good), inches = 0.5,
    xlab = "Distance (yards)", ylab = "Estimated probability")
  curve(expr = predict(object = mod.fit1, newdata = data.frame(distance = x),
    type = "response"), lty = 1, lwd = 2, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit3, newdata = data.frame(distance = x),
    type = "response"), lty = 2, lwd = 1, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit.ps, newdata = data.frame(distance = x),
    type = "response"), lty = 6, lwd = 1, add = TRUE, xlim = c(18,66)) #dot dot dash
  legend(x = 20, y = 0.4, legend = c("First order", "Third order", "Smoothing spline"),
    lty = c(1,2,6), lwd = c(2,1,1), bty = "n")
  # dev.off()  # Create plot for book


  # Additional models
  curve(expr = predict(object = mod.fit.bs, newdata = data.frame(distance = x),
    type = "response"), lty = 3, lwd = 2, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit.ns, newdata = data.frame(distance = x),
    type = "response"), lty = 4, lwd = 2, add = TRUE, xlim = c(18,66)) #dot dot dash

  predict(object = mod.fit.ps, newdata = data.frame(distance = seq(from = 20, to = 60, by = 10)),
    type = "response", se.fit = TRUE)

  # Estimate odds ratio
  library(package = emmeans)

  # Just one odds ratio
  calc.est  <- emmeans(object = mod.fit.ps, specs = ~ distance,
    at = list(distance = c(20,30)))
  confint(object = contrast(object = calc.est , method = "consec", reverse = TRUE),
   level = 0.95, type = "response")

  # Odds ratios for 10-yard increments
  calc.est  <- emmeans(object = mod.fit.ps, specs = ~ distance,
    at = list(distance = c(20,30,40,50,60)))
  confint(object = contrast(object = calc.est, method = "consec", reverse = TRUE),
   level = 0.95, type = "response", adjust = "none")


  # Plot smoothing spline model and include B-spline and natural spline with similar
  #   degrees of freedom
  dev.new(width = 7, height = 6, pointsize = 12)
  symbols(x = w$distance, y = w$good/n$good, circles = sqrt(n$good), inches = 0.5,
    xlab = "Distance (yards)", ylab = "Estimated probability")
  curve(expr = predict(object = mod.fit1, newdata = data.frame(distance = x),
    type = "response"), lty = 1, lwd = 2, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit3, newdata = data.frame(distance = x),
    type = "response"), lty = 2, lwd = 1, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit.ps, newdata = data.frame(distance = x), type = "response"),
    col = "green", add = TRUE, from = 18, to = 66)

  knots4 <- seq(from = 25, to = 55, by = 10)

  mod.fit.bs <- glm(formula = good ~ bs(distance, df = 3), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.bs)
  curve(expr = predict(object = mod.fit.bs, newdata = data.frame(distance = x), type = "response"),
    col = "blue", add = TRUE, from = 18, to = 66)

  # Essentially the same fit as mod.fit.ps
  mod.fit.ns <- glm(formula = good ~ ns(distance, df = 3), family = binomial(link = logit),
    data = placekick)
  summary(object = mod.fit.ns)
  curve(expr = predict(object = mod.fit.ns, newdata = data.frame(distance = x), type = "response"),
    col = "red", add = TRUE, from = 18, to = 66)



#####################################################################
# gam and VGAM packages

  library(package = gam)

  # Intercept only model - Need for a LRT
  mod.fit.int <- glm(formula = good ~ 1, family = binomial(link = logit),
    data = placekick)

  knots4 <- seq(from = 25, to = 55, by = 10)


  # B-spline
  mod.fit.gp <- gam(formula = good ~ bs(x = distance, knots = knots4), family = binomial(link = logit),
    data = placekick)
  class(mod.fit.gp)
  methods(class = "Gam")
  summary(mod.fit.gp, test = "Chisq")
  AIC(mod.fit.gp)

  # Anova(mod.fit.gp)  # Does not work
  anova(mod.fit.int, mod.fit.gp, test = "Chisq")  # Matches Anova(mod.fit.bs)

  # Smoothing spline, uses df = 4 by default and does not automatically determine
  #   the best degrees of freedom, unlike mgcv
  mod.fit.gps <- gam(formula = good ~ s(x = distance), family = binomial(link = logit),
    data = placekick)
  summary(mod.fit.gps)

  # Local regression
  mod.fit.glo <- gam(formula = good ~ lo(x = distance), family = binomial(link = logit),
    data = placekick)
  summary(mod.fit.glo)


  dev.new(width = 7, height = 6, pointsize = 12)
  symbols(x = w$distance, y = w$good/n$good, circles = sqrt(n$good), inches = 0.5,
    xlab="Distance (yards)", ylab="Estimated probability", panel.first = grid())
  curve(expr = predict(object = mod.fit1, newdata = data.frame(distance = x),
    type = "response"), lty = 1, lwd = 2, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit.gp, newdata = data.frame(distance = x),
    type = "response"), lty = 3, lwd = 2, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit.gps, newdata = data.frame(distance = x),
    type = "response"), lty = 1, lwd = 1, add = TRUE, xlim = c(18,66))
  curve(expr = predict(object = mod.fit.glo, newdata = data.frame(distance = x),
    type = "response"), lty = 2, lwd = 1, add = TRUE, xlim = c(18,66))
  legend(x = 20, y = 0.4, legend = c("First order", "B-spline", "Smoothing spline",
    "Local"), lty = c(1,3,1,2), lwd = c(2,2,1,1), bty = "n")


  library(package = VGAM)

  # First-order model (no spline)
  mod.fit.v <- vglm(formula = good ~ distance, family = binomialff,
    data = placekick)
  summary(mod.fit.v)
  Anova(mod.fit.v)

  # B-spline
  mod.fit.v.bs <- vglm(formula = good ~ bs(x = distance, knots = knots4),
    family = binomialff, data = placekick)
  summary(mod.fit.v.bs)
  Anova(mod.fit.v.bs)

  # Smoothing spline
  mod.fit.v.ps <- vgam(formula = good ~ sm.ps(x = distance), data = placekick,
    family = binomialff)
  summary(mod.fit.v.ps)


































#
